<?php

namespace JimsCustomerBooking\Shortcodes;

use JimsCustomerBooking\Admin\BackendPages\NativeFormSettingsPage;
use JimsCustomerBooking\Helpers\OptionHelper;
use JimsCustomerBooking\Helpers\StringHelper;
use JimsCustomerBooking\Template;

class BookingFormShortCode extends Shortcode
{

    /**
     * @var string
     */
    protected $template = 'shortcode.booking_form';

    protected $default_attributes = [];

    /**
     * Order matters, don't change the order if you don't know what you are doing
     * @var array
     */
    protected $step_templates = [
//        'partial.booking_form-step_divisions',
        'partial.booking_form-step_services',
        'partial.booking_form-step_client_info',
    ];

    /**
     * @inheritDoc
     */
    public function Configuration()
    {
        return [
            'shortcode_id' => StringHelper::GetPrefixed('booking_form', 'machine'),
            'label' => 'Booking Form',
            'template' => $this->template
        ];
    }

    /**
     * @inheritDoc
     */
    public function Handler($atts, $content)
    {
        $component_id = mt_rand(111, 9999);
        $css_classes = [ 'wpcb', 'wpcb-form__native', $this->shortcode_id . '-' . $component_id ];
        $customize_options = $this->getCustomizeOption('customer_booking_customizer_options', []);

        $_atts = $this->GetAtts([
            'division_id' => $this->getFormOption('bookingFormDivisionId'),
            'phone_number' => $this->getFormOption('phone'),
            'address_text' => $this->getFormOption('bookingFormAddressText'),
            'details_text' => $this->getFormOption('bookingFormDetailsText'),
            'description_text' => $this->getFormOption('bookingFormDescriptionText'),
            'template' => $this->template,
            'service_ids' => [],
            'component_id' => $component_id,
            'logo_url' =>  apply_filters( StringHelper::getPrefixed('booking_form_logo_url'), $this->getLogoUrl() ),
            'hide_logo' => 0,
            'contained' => 0,
            'show_country' => 1,
            'show_info_text' => 1,
            'images_base_url' => JIMS_CBP_URL . 'assets/dist/img',
            'class' => implode(' ', $css_classes),
            'prefix_class' => StringHelper::getPrefix (StringHelper::PREFIXED_TYPE_SLUG),
            'button_text' => 'Book Now',
            'enable_popup' => 0,
            'show_only_services' => [],
            'exclude_services' => [],
            'grouping_enabled' => isset($customize_options['categories_enabled']) && $customize_options['categories_enabled'] == true ? 1 : 0,
            'custom_order_enabled' => isset($customize_options['custom_order']) && $customize_options['custom_order'] == true ? 1 : 0,
            'alphabetical_order' => isset($customize_options['alphabetical_order']) && $customize_options['alphabetical_order'] == true ? 1 : 0,
        ]);

        if(isset($atts['class']) && !empty($atts['class'])){
            $atts['class'] =  implode(' ', $css_classes) . ' ' . $atts['class'];
        }

        if (isset($atts['service_ids']) && !empty($atts['service_ids'])) {
            $atts['service_ids'] = explode(',', $atts['service_ids']);
        }

        // apply filters, modify attributes
        $atts = apply_filters (
            StringHelper::getPrefixed ('booking_form_atts'),
            shortcode_atts($_atts, $atts, $this->shortcode_id ),
            $content, $this->shortcode_id);


        if (isset($atts['contained']) && $atts['contained'] == 1) {
            $atts['class'] .= ' contained';
        }

        if (isset($atts['enable_popup']) && $atts['enable_popup'] == 1) {
            //jims-popup-enabled
            $atts['class'] .= ' jims-popup-enabled';
        }

        //show only selected services
        if (isset($atts['show_only_services']) && !empty($atts['show_only_services'])) {
            $atts['show_only_services'] = explode(',', $atts['show_only_services']);
        }

        //exclude services
        if (isset($atts['exclude_services']) && !empty($atts['exclude_services'])) {
            $atts['exclude_services'] = explode(',', $atts['exclude_services']);
        }

        // load steps
        $step_templates = $this->loadSteps();
        $filter_names = [];
        if(count($step_templates) < count($this->step_templates)){
            $steps_html = '';
            error_log('step_templates count is less than step_templates count');
        } else {
            $steps_html = $this->renderSteps($step_templates, $atts, $filter_names);
        }

        // add steps to attributes
        $atts['steps'] = $steps_html;

        // get template object, it contains override if the shortcode has template attribute
        $template = new Template($atts['template']);

        // render template, allows overriding the html
        return apply_filters (
            StringHelper::GetPrefixed ('form_html'),
            $template->get( $atts + [ 'content' => $content] ),
            $atts, $content, $this->shortcode_id
        );
    }

    public function getFormOption($name, $default = ''){
        $options = OptionHelper::get(NativeFormSettingsPage::$option_name, [] );
        return $options[$name] ?? $default;
    }

    public function getCustomizeOption($name, $default = ''){
        $option = OptionHelper::get($name, []);
        return $option === false ? $default : $option;
    }

    /**
     * @param $step_templates Template[]
     * @return string
     */
    public function renderSteps($step_templates, $params = [], &$filter_names = []){
        $html = '';
        foreach ($step_templates as $index => $step){
            $filter_names[] = $filter_name = StringHelper::getPrefixed('booking_form_step_'.$step->getName().'_html', 'machine');
            $html .= apply_filters($filter_name,$step->get($params), $this->shortcode_id, $index);
        }

        $html = apply_filters(
            StringHelper::getPrefixed('booking_form_steps_html'),
            $html,
            $this->shortcode_id
        );

        return $html;
    }

    /**
     * @return Template[]
     */
    public function loadSteps(){
        $this->step_templates = apply_filters(
            StringHelper::getPrefixed('booking_form_steps'),
            $this->step_templates,
            $this->shortcode_id
        );

        return array_map(function($template_name){
            return new Template($template_name);
        }, $this->step_templates);
    }

    /**
     * @return string
     */
    public function getLogoUrl(): string {
        $logo = '';

        if( \has_custom_logo() ) {
            $custom_logo_id = get_theme_mod( 'custom_logo' );
            $image = wp_get_attachment_image_src( $custom_logo_id , 'full' );
            $logo = is_array ( $image ) && count ( $image ) > 0 ? $image[0] : '';
        }

        if( empty($logo) ){
            $logo = JIMS_CBP_URL . 'assets/dist/img/jims-logo.png';
        }

        return $logo;
    }
}